import jsonld from "jsonld";
import { Parser } from "n3";
export class Item {
    id;
    idx;
    item_set;
    factory;
    thing;
    item;
    lastState = "";
    cached = [];
    constructor(id, idx, factory, item, thing, item_set) {
        this.id = id;
        this.idx = idx;
        this.factory = factory;
        this.item = item;
        this.thing = thing;
        this.item_set = item_set;
    }
    async getQuads(deep = false, reverse = true, done = new Set()) {
        if (done.has(this.idx)) {
            return [];
        }
        done.add(this.idx);
        const json = Object.assign({}, this.thing);
        const jsonStr = JSON.stringify(json);
        const quads = [];
        if (jsonStr === this.lastState) {
            quads.push(...this.cached);
        }
        else {
            if (!reverse) {
                delete json["@reverse"];
            }
            const nquads = await jsonld.toRDF(json, {
                format: "application/n-quads",
            });
            const theseQuads = new Parser()
                .parse(nquads)
                .filter((x) => !x.predicate.value.startsWith("http://omeka.org/s/vocabs/o#"));
            quads.push(...theseQuads);
            this.cached = theseQuads;
        }
        if (deep) {
            for (const value of Object.values(this.item)) {
                const values = Array.isArray(value) ? value : [value];
                for (const v of values) {
                    if (v instanceof Item) {
                        quads.push(...(await v.getQuads(deep, reverse, done)));
                    }
                }
            }
        }
        return quads;
    }
    async save() {
        const payload = this.factory.transformItem(this.item, this.item_set);
        const resp = await this.factory.config.put("items/" + this.idx, payload);
        const json = await resp.json();
        this.thing = json;
    }
}
